@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
-__version__ = '1.4.9'
+__version__ = '1.5.5'
 
 from .app import Kola
 from .blueprint import Blueprint
-from .utils import safe_call
+from .utils import safe_call, safe_func
 from .log import logger
