@@ -2,6 +2,8 @@
 
 import functools
 import time
+from .utils import safe_func
+from events import Events
 
 
 class RoutesMixin(object):
@@ -10,9 +12,11 @@ class RoutesMixin(object):
     """
 
     rule_map = None
+    events = None
 
     def __init__(self):
         self.rule_map = dict()
+        self.events = Events()
 
     def add_route_rule(self, endpoint=None, view_func=None, **options):
         assert view_func is not None, 'expected view func if endpoint is not provided.'
@@ -27,9 +31,8 @@ class RoutesMixin(object):
 
         self.rule_map[endpoint] = view_func
 
-    def route(self, **options):
+    def route(self, endpoint=None, **options):
         def decorator(f):
-            endpoint = options.pop('endpoint', None)
             self.add_route_rule(endpoint, f, **options)
             return f
         return decorator
@@ -39,28 +42,13 @@ class RoutesMixin(object):
 
 
 class AppCallBacksMixin(RoutesMixin):
-    create_conn_func_list = None
-    before_first_request_func_list = None
-    before_request_func_list = None
-    after_request_func_list = None
-    before_response_func_list = None
-    close_conn_func_list = None
-
-    def __init__(self):
-        super(AppCallBacksMixin, self).__init__()
-        self.create_conn_func_list = []
-        self.before_first_request_func_list = []
-        self.before_request_func_list = []
-        self.after_request_func_list = []
-        self.before_response_func_list = []
-        self.close_conn_func_list = []
 
     def create_conn(self, f):
         """
         连接建立成功后
         f(conn)
         """
-        self.create_conn_func_list.append(f)
+        self.events.create_conn += safe_func(f)
         return f
 
     def before_first_request(self, f):
@@ -68,7 +56,7 @@ class AppCallBacksMixin(RoutesMixin):
         第一个请求，请求解析为json成功后
         f(request)
         """
-        self.before_first_request_func_list.append(f)
+        self.events.before_first_request += safe_func(f)
         return f
 
     def before_request(self, f):
@@ -76,23 +64,23 @@ class AppCallBacksMixin(RoutesMixin):
         请求解析为json成功后
         f(request)
         """
-        self.before_request_func_list.append(f)
+        self.events.before_request += safe_func(f)
         return f
 
     def after_request(self, f):
         """
         执行完route对应的view_func后
-        f(request)
+        f(request, exc)
         """
-        self.after_request_func_list.append(f)
+        self.events.after_request += safe_func(f)
         return f
 
     def before_response(self, f):
         """
         在 stream.write 之前，传入encode之后的data
-        f(response)
+        f(conn, response)
         """
-        self.before_response_func_list.append(f)
+        self.events.before_response += safe_func(f)
         return f
 
     def close_conn(self, f):
@@ -100,37 +88,18 @@ class AppCallBacksMixin(RoutesMixin):
         连接close之后
         f(conn)
         """
-        self.close_conn_func_list.append(f)
+        self.events.close_conn += safe_func(f)
         return f
 
 
 class BlueprintCallBacksMixin(RoutesMixin):
-    before_request_func_list = None
-    after_request_func_list = None
-    create_app_conn_func_list = None
-    before_app_first_request_func_list = None
-    before_app_request_func_list = None
-    after_app_request_func_list = None
-    before_app_response_func_list = None
-    close_app_conn_func_list = None
-
-    def __init__(self):
-        super(BlueprintCallBacksMixin, self).__init__()
-        self.before_request_func_list = []
-        self.after_request_func_list = []
-        self.create_app_conn_func_list = []
-        self.before_app_first_request_func_list = []
-        self.before_app_request_func_list = []
-        self.after_app_request_func_list = []
-        self.before_app_response_func_list = []
-        self.close_app_conn_func_list = []
 
     def before_request(self, f):
         """
         请求解析为json成功后
         f(request)
         """
-        self.before_request_func_list.append(f)
+        self.events.before_request += safe_func(f)
         return f
 
     def after_request(self, f):
@@ -138,7 +107,7 @@ class BlueprintCallBacksMixin(RoutesMixin):
         执行完route对应的view_func后
         f(request)
         """
-        self.after_request_func_list.append(f)
+        self.events.after_request += safe_func(f)
         return f
 
     def create_app_conn(self, f):
@@ -146,7 +115,7 @@ class BlueprintCallBacksMixin(RoutesMixin):
         连接建立成功后
         f(conn)
         """
-        self.create_app_conn_func_list.append(f)
+        self.events.create_app_conn += safe_func(f)
         return f
 
     def before_app_first_request(self, f):
@@ -154,7 +123,7 @@ class BlueprintCallBacksMixin(RoutesMixin):
         第一次请求，请求解析为json成功后
         f(request)
         """
-        self.before_app_first_request_func_list.append(f)
+        self.events.before_app_first_request += safe_func(f)
         return f
 
     def before_app_request(self, f):
@@ -162,7 +131,7 @@ class BlueprintCallBacksMixin(RoutesMixin):
         请求解析为json成功后
         f(request)
         """
-        self.before_app_request_func_list.append(f)
+        self.events.before_app_request += safe_func(f)
         return f
 
     def after_app_request(self, f):
@@ -170,15 +139,15 @@ class BlueprintCallBacksMixin(RoutesMixin):
         执行完route对应的view_func后
         f(request)
         """
-        self.after_app_request_func_list.append(f)
+        self.events.after_app_request += safe_func(f)
         return f
 
     def before_app_response(self, f):
         """
         在 stream.write 之前，传入encode之后的data
-        f(response)
+        f(conn, response)
         """
-        self.before_app_response_func_list.append(f)
+        self.events.before_app_response += safe_func(f)
         return f
 
     def close_app_conn(self, f):
@@ -186,5 +155,5 @@ class BlueprintCallBacksMixin(RoutesMixin):
         连接close之后
         f(conn)
         """
-        self.close_app_conn_func_list.append(f)
+        self.events.close_app_conn += safe_func(f)
         return f
