@@ -20,13 +20,9 @@ class Connection(object):
 
         self.stream.set_close_callback(self._on_connection_close)
 
-        # create_conn_func_list
-        for func in self.app.create_conn_func_list:
-            safe_call(func, self)
-        # 执行blueprints
+        self.app.events.create_conn(self)
         for name, bp in self.app.blueprints.items():
-            for func in bp.create_app_conn_func_list:
-                safe_call(func, self)
+            bp.events.create_app_conn(self)
 
     def write(self, data, callback=None):
         """
@@ -35,13 +31,9 @@ class Connection(object):
         if isinstance(data, dict):
             data = self.make_rsp(**data)
 
-        # before_app_response_func_list
-        for func in self.app.before_response_func_list:
-            safe_call(func, data)
-        # 执行blueprints
+        self.app.events.before_response(self, data)
         for name, bp in self.app.blueprints.items():
-            for func in bp.before_app_response_func_list:
-                safe_call(func, data)
+            bp.events.before_app_response(self, data)
 
         if not self.stream.closed():
             self._write_callback = callback
@@ -91,13 +83,9 @@ class Connection(object):
             self._close_callback = None
             safe_call(callback)
 
-        # 执行blueprints
         for name, bp in self.app.blueprints.items():
-            for func in bp.close_app_conn_func_list:
-                safe_call(func, self)
-        # close_conn_func_list
-        for func in self.app.close_conn_func_list:
-            safe_call(func, self)
+            bp.events.close_app_conn(self)
+        self.app.events.close_conn(self)
 
     def _read_message(self):
         if not self.stream.closed():
@@ -151,26 +139,18 @@ class Connection(object):
             request.write(dict(ret=constants.RET_SYSTEM, error=error))
             return None
 
-        # before_first_request_func_list
         if not self.app.got_first_request:
             self.app.got_first_request = True
-            for func in self.app.before_first_request_func_list:
-                safe_call(func, request)
-            # 执行blueprints
-            for name, bp in self.app.blueprints.items():
-                for func in bp.before_app_first_request_func_list:
-                    safe_call(func, request)
+            self.app.events.before_first_request(request)
 
-        # before_request_func_list
-        for func in self.app.before_request_func_list:
-            safe_call(func, request)
-        # 执行blueprints
+            for name, bp in self.app.blueprints.items():
+                bp.events.before_app_first_request(request)
+
+        self.app.events.before_request(request)
         for name, bp in self.app.blueprints.items():
-            for func in bp.before_app_request_func_list:
-                safe_call(func, request)
+            bp.events.before_app_request(request)
         if request.blueprint:
-            for func in request.blueprint.before_request_func_list:
-                safe_call(func, request)
+            request.blueprint.events.before_request(request)
 
         view_func_exc = None
         view_func_result = None
@@ -185,15 +165,12 @@ class Connection(object):
             request.write(dict(ret=constants.RET_SYSTEM,
                                error=error if self.app.debug else constants.ERROR_INTERNAL))
 
-        # after_request_func_list
         if request.blueprint:
-            for func in request.blueprint.after_request_func_list:
-                safe_call(func, request, view_func_exc or view_func_result)
-        # 执行blueprints
+            request.blueprint.events.after_request(request, view_func_exc or view_func_result)
+
         for name, bp in self.app.blueprints.items():
-            for func in bp.after_app_request_func_list:
-                safe_call(func, request, view_func_exc or view_func_result)
-        for func in self.app.after_request_func_list:
-            safe_call(func, request, view_func_exc or view_func_result)
+            bp.events.after_app_request(request, view_func_exc or view_func_result)
+
+        self.app.events.after_request(request, view_func_exc or view_func_result)
 
         return view_func_result
