@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import functools
 from .log import logger
 
 
@@ -10,6 +11,16 @@ def safe_call(func, *args, **kwargs):
     try:
         return func(*args, **kwargs)
     except Exception, e:
-        logger.exception('func raise exception. func: %s', func)
+        logger.error('exc occur. e: %s, func: %s', e, func, exc_info=True)
         # 调用方可以通过 isinstance(e, BaseException) 来判断是否发生了异常
         return e
+
+
+def safe_func(func):
+    """
+    把函数变为安全的
+    """
+    @functools.wraps(func)
+    def func_wrapper(*args, **kwargs):
+        return safe_call(func, *args, **kwargs)
+    return func_wrapper
